;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ Filename: JTC_VGA_TILE_SPRITE_v00x.s
;/
;/ Author: JT Cook - http://www.avalondreams.com
;/ 
;/ Description: 210x192 tile/sprite driver (32x32 scrolling tile map)
;/ Supports 8 16x16 sprites and 8-way scrolling
;/ Does not use palette, only reads colors from tiles and sprites
;/ Stripped out Josh's VGA driver and reworked the rendering the code
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ INCLUDES 
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; Include our register and chip definitions file
.include "p24HJ256GP206.inc"

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ DEFINES
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;----------------------------
;Macro to simulate fake classic scanlines
.macro Draw_Odd_Scanline
        mov.b [W7++], [W8]                          ; (1) Copy out of the previously filled scanline buffer to PORTB
;        mov.b W12, [W8]                          ; (1) Copy W12 (black) to PORTB
.endm

; width and height of screen 
;.equ VGA_SCREEN_WIDTH,     210
.equ VGA_SCREEN_HEIGHT,    240

; controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
.equ VGA_VERTICAL_OFFSET,  34

; size of sprite pointer
.equ Sprite_Ptr_Size,    0x08    

; number of bits each pixel is encoded with
.equ VGA_PIXELS_PER_BYTE,  1            

; Number of clocks per pixel (10 in the case of VGA) Modifying this won't really change anything,
; you will need to change code delays to get the correct clocks per pixel change
.equ CLOCKS_PER_PIXEL, 6 

; Number of lines we should repeat to reduce the vertical resolution
.equ LINE_REPT_COUNT,   2

; Mask to determine whether sprite bit is on
.equ SPRITE_ON_MASK,        0x0001

; If you want to add more variables to the sprite record make sure you increase
; the sprite struct size (in bytes) defined below
.equ SPRITE_STRUCT_SIZE, 8

; VGA Port bits for enabling and disabling syncs. These are used on PORTB only.
; -----------------------PORTB -------------------
;   7      6     5     4     3     2     1     0
; Vsync  Hsync   B1    B0    G1    G0    R1    R0
.equ VGA_HSYNC_BIT,     6
.equ VGA_VSYNC_BIT,     7

;----------------------------
;Macro to draw sprite before rendering starts
; First run is 65 cycles, 2nd and runs after that are 64 cycles
.macro Draw_Sprite
        mov.b [W2+4], W4 ;(1) get Y value for sprite
        sub W5, W4, W4   ;(1) scanline - sprite Y (should set overflow)
	    BRA n, 1f ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
		BRA GT, 2f  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W4  ;(1) Multiply by 16 since each sprite line is 16 pixels
        mov W6, W3 ;(1) get adddress to sprite scanline
		;nop;debug
		and #0xFF, W1     ;(1) ***New*** Clear out upper 8 bits
        mov.b [W2+2], W1 ;(1) get x value for sprite
        add W3, W1, W3    ;(1) add x value to sprite scanline 
		mov [W2+6], W1      ;(1) sprite graphic address
        add W1, W4, W1     ;(1) add scanline offset to get correct sprite line
        ;unrolled loop for drawing sprite - 50 cycles (including goto)
        ;pixel 0
		;W3 old scanline buffer for NTSC driver, W6 scanline buffer for VGA
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+0]  ;(1)
        ;pixel 1
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+1]  ;(1)
        ;pixel 2
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+2]  ;(1)
        ;pixel 3
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+3]  ;(1)
        ;pixel 4
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+4]  ;(1)
        ;pixel 5
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+5]  ;(1)
        ;pixel 6
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+6]  ;(1)
        ;pixel 7
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+7]  ;(1)
        ;pixel 8
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+8]  ;(1)
        ;pixel 9
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+9]  ;(1)
        ;pixel 10
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+10]  ;(1)
        ;pixel 11
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+11]  ;(1)
        ;pixel 12
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+12]  ;(1)
        ;pixel 13
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+13]  ;(1)
        ;pixel 14
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+14]  ;(1)
        ;pixel 15
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+15]  ;(1)
0: ;first skip
        goto 3f     ;(2) 
;-- Time to kill if we aren't drawing the second or 4th sprite
1:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
        repeat #(64-5-5)    ;(1)
        nop               ;(1+n)
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
	goto 3f ;(2)
2:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
        repeat #(64-7-4)    ;(1)
        nop               ;(1+n)
	goto 3f ;(2)
3:
        add #Sprite_Ptr_Size, W2        ;(1) advance sprite pointer to get info for this sprite
.endm
;----------------------------
;Macro to draw sprite while rendering VGA signal (108 cycles)
; 5 cycles a pixel, VGA signal is on 4th cycle
; 21 Pixels rendered
.macro Draw_Sprite_Signal
        mov.b [W2+4], W4 ;(1) get Y value for sprite
        sub W5, W4, W4   ;(1) scanline - sprite Y (should set overflow)
		nop				;(1)
	Draw_Odd_Scanline ;(1) - Macro		
	    BRA n, 1f ;(1/2) if sprite Y is greater than current scanline, then don't draw sprite
       ;Sprite Y is less than or equal to Y scanline (possible sprite drawing)
        cp W4, #15          ;(1) if it greater than 15, then the sprite is not on this scanline        
		BRA GT, 2f  ;(1/2) if it is greater than 15, sprite is not on this scanline
       ; - find the correct sprite scanline value - 6 cycles
        sl W4, #4, W4  ;(1) Multiply by 16 since each sprite line is 16 pixels
	Draw_Odd_Scanline ;(1) - Macro		
        mov W6, W3 ;(1) get adddress to sprite scanline
		;nop;debug
		and #0xFF, W1     ;(1) ***New*** Clear out upper 8 bits
        mov.b [W2+2], W1 ;(1) get x value for sprite
        add W3, W1, W3    ;(1) add x value to sprite scanline 
	Draw_Odd_Scanline ;(1) - Macro		
		mov [W2+6], W1      ;(1) sprite graphic address
        add W1, W4, W1     ;(1) add scanline offset to get correct sprite line
		nop				;(1)
		nop				;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
	;8 Extra cycles for rendering the VGA signal
    ;32 Extra cycles after this point for a total of 40 (2 extra cycles for each pixel)
        ;unrolled loop for drawing sprite - 50 cycles (including goto)
        ;pixel 0
		;W3 old scanline buffer for NTSC driver, W6 scanline buffer for VGA
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+0]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 1
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+1]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 2
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+2]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 3
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+3]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 4
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+4]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 5
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+5]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 6
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+6]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 7
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+7]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 8
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+8]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 9
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+9]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 10
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+10]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 11
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+11]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 12
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+12]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 13
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+13]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 14
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+14]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
        ;pixel 15
        mov.b [W1++], W4  ;(1) get the sprite color
        cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        mov.b W4, [W3+15]  ;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		;nop				;(1)
0: ;first skip
        goto 3f     ;(2) 
;-- Time to kill if we aren't drawing the second or 4th sprite
1:  ;No sprite, 1st check (Sprite Y greater/lower than current scanline)
		nop; 
		nop;
	Draw_Odd_Scanline ;(1) - Macro		
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
.rept 17
		nop;
		nop;
		nop;
	Draw_Odd_Scanline ;(1) - Macro		
		nop;
.endr
		nop;
		nop;
		nop;
	Draw_Odd_Scanline ;(1) - Macro		

	    ;cpseq W4, W10 ;(1/2) if pixel is not zero, then draw it to scanline buffer
        ;repeat #(104-5-5-5)    ;(1)
        ;nop               ;(1+n)
        ;mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
		;add W7, #19, W7   ;(1)		
	goto 3f ;(2)
2:  ;No sprite, 2nd check (Passed 1st check, but sprite not being drawn on this scanline)
	Draw_Odd_Scanline ;(1) - Macro		
        mov #0, W4        ;(1)  clear out upper 8 bits of W4 register
.rept 17 
		nop;
		nop;
		nop;
	Draw_Odd_Scanline ;(1) - Macro		
		nop;
.endr
		nop;
		nop;
		nop;
	Draw_Odd_Scanline ;(1) - Macro		

        ;repeat #(104-6-5-5)    ;(1)
        ;nop               ;(1+n)
		;add W7, #19, W7   ;(1)		
	goto 3f ;(2)
3:
        add #Sprite_Ptr_Size, W2        ;(1) advance sprite pointer to get info for this sprite
		nop				;(1)
	Draw_Odd_Scanline ;(1) - Macro		
		nop				;(1)
.endm

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ EXTERNALS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.extern _g_TileMapBasePtr       ; 16-bit pointer to tile map, allows indirection
.extern _g_SpriteList           ; 16-bit pointer to sprite table records
.extern _g_TileSet              ; 16-bit pointer to 8x8 tile bitmaps, allows indirection
.extern _g_MapScroll

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ SRAM SECTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following section describes initialized (data) near section. The reason we want
; to place these variables into the near SRAM area is so that ASM codes, that don't use many
; bits to encode data, will be able to access the variables.
.section *,data,near
_g_CurrentLine: .int 0                  ; Line count 0->524, initalize higher than active so it resets its variables immediately
ReptLine:       .int LINE_REPT_COUNT    ; Repeated line counter 0->2
TILEINDPTR:     .int 0                  ; Current tile we are drawing
LINECOUNT:      .int 0                  ; Line counter increments by 8 bytes 0->56
_g_VsyncFlag:   .int 0                  ; Flag to indicate we are in v-sync
_g_VscrollFine: .int 0                  ; Fine scrolling flag 0->7
FILLBUFPTR:     .int 0                  ; Pointer to the buffer we want to fill
DRAWBUFPTR:     .int 0                  ; Pointer to the buffer we want to draw to the screen
DUMBYPORT:      .int 0                  ; Dumby port we will draw too during the first two lines since we don't have data queued up yet
_g_TileMapLogicalWidth: .int 32         ; Logical width of tile map
_g_RasterLine:   .int 0                 ; current logical raster line

; Uninitialized data sections
.section *,bss,near
;ScanLineBuf0: .space (VGA_SCREEN_WIDTH + 2*8)   ; Scanline buffer is 22+2 tiles at 8 bytes = 216 bytes
;ScanLineBuf1: .space (VGA_SCREEN_WIDTH + 2*8)   ; Scanline buffer is 25+2 tiles at 8 bytes = 216 bytes
ScanLineBuf0: .space (256+16)   ; Scanline buffer 
ScanLineBuf1: .space (256+16)   ; Scanline buffer 

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ GLOBAL EXPORTS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.global __T1Interrupt               ; Declare Timer 1 ISR name global
.global _g_VsyncFlag                ; int, read, Flag to indicate we are in v-sync
.global _g_VscrollFine              ; int, read/write, Fine scrolling flag 0->7
.global _g_CurrentLine              ; int, read only, current physical line 0...524
.global _g_RasterLine               ; int, read only, current logical raster line
.global _g_TileMapLogicalWidth      ; int, read/write, logical width of tile map

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ ASM FUNCTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following are code sections
.text           

__T1Interrupt:

        ; Our instruction clock is running at 2*VGA dot clock
        ; so every 2 clock cycles is a normal 640x480 pixel
        disi #0x3FFF                                ; (1) Disable interupts for up to 10000 cycles

        ; Save off variables we will use
        push.s                                      ; (1) Push shadow registers
        push W0                                     ; (1) Temp variable
        push W1                                     ; (1) Temp variable
        push W2                                     ; (1) Temp variable
        push W3                                     ; (1) Temp variable
        push W4                                     ; (1) Temp variable
        push W5                                     ; (1) Temp variable
        push W6                                     ; (1) Temp variable
        push W7                                     ; (1) Temp variable
        push W8                                     ; (1) Temp variable
        push W9                                     ; (1) Temp variable
		push W10									; (1) Temp variable
		push W11									; (1) Temp variable
		push W12									; (1) Temp variable
        push RCOUNT                                 ; (1) Push rcount so we can use repeat loops

        ; Increment which line we are on
        inc _g_CurrentLine                          ; (1)
        mov _g_CurrentLine, W0                      ; (1) Set our line variable into W0

        ; Test if we are done with this frame (525 lines)
        mov #524, W2                                ; (1)
        cp W0, W2                                   ; (1) See if we are at line 524 = 0
        bra nz, DONE_LINE_TEST                      ; (1/2) if(_g_CurrentLine != 524) jump
        setm _g_CurrentLine                         ; (1) Reset Line counter set to 0xFFFF
DONE_LINE_TEST: 

        ; Total = 12

        ; The following section of code determines what kind of line we should be 
        ; drawing to the screen. Is it a Vsync Line, Black Top/Bottom, or an
        ; Active Video line. All of this is determined by the _g_CurrentLine counter
        ; variable

        ; if(_g_CurrentLine < VSYNC_LINE_COUNT)
        mov #1, W2                                  ; (1)
        cp W0, W2                                   ; (1) Compare to 1
        bra gt, NOT_VSYNC_LINE                      ; (1/2) if(Line > 1) jump

        ; Vsync line (17)
        repeat  #9                                  ; (1)
        nop                                         ; (1+n)
        bset LATB, #VGA_VSYNC_BIT                   ; (1) turn vsync bit on
        goto DRAW_BLACK_LINE                        ; (2) draw a black line to the screen

NOT_VSYNC_LINE:

        ; Test to see if this is a top blanking line
        mov #VGA_VERTICAL_OFFSET, W2                ; (1)
        cp W0, W2                                   ; (1) Compare to 33
        bra ge, NOT_TOP_BLANK_LINE                  ; (1/2) if(Line > 33) jump

        ; Blank Line Top (17)
        repeat  #2                                  ; (1)
        nop                                         ; (1+n)
        clr _g_RasterLine                           ; (1) reset rasterline count
        mov _g_TileMapBasePtr, W0                   ; (1) Copy Tile map start pointer to W0
        mov W0, TILEINDPTR                          ; (1) Set Tile Index Pointer
        bclr LATB, #VGA_VSYNC_BIT                   ; (1) turn vsync bit off
        goto DRAW_BLACK_LINE                        ; (2) draw a black line to the screen

NOT_TOP_BLANK_LINE:

        ; if(_g_CurrentLine > LAST_ACTIVE_LINE)
        mov #((VGA_SCREEN_HEIGHT*LINE_REPT_COUNT)+VGA_VERTICAL_OFFSET), W2 ; (1) Bottom line
        cp W0, W2                                   ; (1) Compare to 514
        bra lt, NOT_BOTTOM_BLANK_LINE               ; (1/2) if(Line < 514) jump

        ; Blank Line Bottom (17)
        nop                                         ; (1) delay
        nop                                         ; (1) delay
        bclr LATB, #VGA_VSYNC_BIT                   ; (1) turn vsync bit off
        setm _g_VsyncFlag                           ; (1) set the vsync flag 
        goto DRAW_BLACK_LINE                        ; (2) draw a black line to the screen

NOT_BOTTOM_BLANK_LINE:

        ; This must be an active line (17)
        bclr LATB, #VGA_VSYNC_BIT                   ; (1) turn vsync bit off
        clr _g_VsyncFlag                            ; (1) g_VsyncFlag = 0
        nop                                         ; (1) delay
        goto DRAW_ACTIVE_LINE                       ; (2) draw an active line to the screen



; ----------------- BLACK LINE -------------------------
DRAW_BLACK_LINE:

        ; This draws a black line with h-sync (used for v-sync and blanking lines)
        bset LATB, #VGA_HSYNC_BIT                   ; (1) turn hsync bit on

        ; Hold Hsync for 190 clocks total
        repeat #187                                 ; (1)
        nop                                         ; (1+n)

        bclr LATB, #VGA_HSYNC_BIT                   ; (1) turn hsync bit off and set to black RGB

        ; Done with this completely
        goto FINISHED_VGA                           ; Dont need to count because interrupts will keep us locked


; ----------------- ACTIVE LINE -------------------------
DRAW_ACTIVE_LINE:

        ; This draws an active line with h-sync, hsync is 190 clock cycles low
        ; ------- HSYNC ON ---------
        bset LATB, #VGA_HSYNC_BIT                   ; (1) turn hsync bit on

        ; Vertical fine scroll offsets first line - determine if we are on the first line
        mov _g_VscrollFine, W3                      ; (1) Copy vertical fine scrolling to W3, NOTE: this only is applicable on the first active scanline
        sl W3, #3, W3                               ; (1) Multiply fine scrolling by 8 since its going to sum into our byte offset

        mov _g_CurrentLine, W1                      ; (1) Compare W1 with line #34 = start of our active lines
        sub #35, W1                                 ; (1)
        bra le, FIRST_SECOND_LINE                   ; (1/2) branch if we are at the first line or second line
        mov #PORTB, W8                              ; (1)
        repeat #2                                   ; (1)
        nop                                         ; (1+n)
        goto DONE_FIRST_SECOND_LINE                 ; (2)
FIRST_SECOND_LINE:
        mov W3, LINECOUNT                           ; (1) Preset Line count
        mov #ScanLineBuf0, W0                       ; (1)
        mov W0, FILLBUFPTR                          ; (1) Start by filling scanlinebuf0
        mov #ScanLineBuf1, W0                       ; (1)
        mov W0, DRAWBUFPTR                          ; (1) Start by drawing out of scanlinebuf1
        mov #DUMBYPORT, W8                          ; (1) Render data to a dumby port on first two lines
DONE_FIRST_SECOND_LINE:

        ; Total = 13

        ; Assign the pointers to where we want to draw and fill
        mov FILLBUFPTR, W6                          ; (1) W6 is the fill pointer always
        mov DRAWBUFPTR, W7                          ; (1) W7 is the draw pointer always

        ; Figure out what type of scanline is this. Since we are line doubling
        ; we actually have two lines of data to fill up just one scanbuffer,
        ; while the second scanbuffer is rendering data out. Thus on even lines
        ; we will fill up the scanbuffer, on odd lines we will add sprites and
        ; other things.
        mov _g_CurrentLine, W1                      ; (1) Get the current line number in W1
        and #0x01, W1                               ; (1) Mask off bottom bit to see if we are odd
        bra z, EVEN_LINE_RENDER                     ; (1/2) If even jump
        ;bra z, ODD_LINE_RENDER                     ; (1/2) If even jump
        goto ODD_LINE_RENDER                        ; (2) Jump to seperate rendering path on odd lines
EVEN_LINE_RENDER:

        ; Total clocks spent in hsync at this point is 22
        ; DO NOT USE OR TOUCH W7 or W8 in here (used for rendering scanline backbuffer to VGA)!!!!

        ; ---------- Rendering Space -------------------- 
        ; Free rendering space

        repeat #(152-26-120-1)                                  ; (1)
        nop                                         ; (1 + n)
        ;W1 holds the tile row counter for screen scrolling (should always be masked with 0x1F)
	;W2 holds current scanline offset for tile graphics
        ;W3 holds current scanline offset for tile map
        ;W4 temp var
	;W5 temp 8 bit var	
        mov _g_MapScroll, W4  			    ; (1) get Y scroll value
        mov.b [W4], W4          		    ; (1) get value for Y
	mov _g_CurrentLine, W2                      ; (1) Line count 0->525
        sub #34, W2                                 ; (1) Line count -= 34 because line 34 = start screen
        asr W2, #1, W2                              ; (1)/2 since we double scanlines
        add W2, W4, W2				    ; (1) add Y scroll to scanline
	mov #0xFF, W5 
	and W2, W5, W2			    ; (1) mask off so we only have max 255 scanline value 
        asr W2, #3, W3                              ; (1) get scanline number for matching tiles
        and W2, #0x7, W2                            ; (1) mask of 3 bits for tile graphics offset
	sl W2, #3, W2                               ; (1) *8 for tile graphics offset
        mov _g_TileSet, W4                          ; (1) get address for tile graphics
        add W4, W2, W2                              ; (1) add tile graphics address, to tile graphics offset
	
        sl W3, #5, W3                               ; (1) *32 for tile map offset (each row is 32 bytes)
        mov _g_TileMapBasePtr, W4                   ; (1) get address for tile map
	add W3, W4, W3				    ; (1) add tile map address to tile map offset
	mov _g_MapScroll, W4                        ; (1) address for MapScroll struct
	add W4, #1, W4                              ; (1) get address for X scroll value
	mov.b [W4], W1				    ; (1) get X scroll value
	and W1, #7, W4				    ; (1) copy and mask X scroll value
        asr W1, #3, W1 				    ; (1) divide X scroll / 8 to get which tile we start at
        mov #16, W5				    ; (1) get value we are subtracting from
	sub W5, W4, W4				    ; (1) use this to determine where we start drawing tiles at
        mov #0, W5                                  ; (1) clear out W5 (used for 8 bit value)	
        ;mov #0, W1                                  ; (1) tile counter(should be masked with 0x1F)
        ; Since the Fill and Draw buffers have a dead zone of 8 pixels to the left side we will skip past that
        ; dead zone by adding 8 to them. This makes it easier to draw sprites if we keep the dead zone.
        ; Render out the scanline
        add W6, W4, W6                                  ; (1) Increment past boarder
        ;add W7, W4, W7                                  ; (1) Increment past boarder
        add #16, W7                                  ; (1) Increment past boarder
;Get Y value, add Y value to scanline, mask

.rept 8 ; 15 cycles
	add W1, W3, W4                              ; (1) add current tile location to tile offset
	mov.b [W4], W5                              ; (1) get tile number       
	sl W5, #6, W4                               ; (1) *64 since each tile is 64 bytes 
        add W4, W2, W4                              ; (1) add tile number to tile offset
	add W1, #1, W1				    ; (1) advance to new tile
	and W1, #0x1F, W1                             ; (1) mask tile number so it is always 0-31
        mov.b [W4++], [W6++]                        ; (2) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
.endr        


        ; ------- HSYNC OFF ---------
        bclr LATB, #VGA_HSYNC_BIT                   ; (1) turn hsync bit off


        ; ------ LEFT BLANKING REGION --------
        ; Now set PORTB rgb components to 0 (we should spend about 80 clock cycles here) 
        clr PORTB                                   ; (1) draw black


        ; ---------- Rendering Space -------------------- 
        ; Free rendering space
        repeat #(72-60)                             ; (1)
        nop                                         ; (1 + n)
.rept 4 ; 15 cycles
	add W1, W3, W4                              ; (1) add current tile location to tile offset
	mov.b [W4], W5                              ; (1) get tile number       
	sl W5, #6, W4                               ; (1) *64 since each tile is 64 bytes 
        add W4, W2, W4                              ; (1) add tile number to tile offset
	add W1, #1, W1				    ; (1) advance to new tile
	and W1, #0x1F, W1                             ; (1) mask tile number so it is always 0-31
        mov.b [W4++], [W6++]                        ; (2) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
.endr        
;1056 clocks = 176 pixels
;.rept VGA_SCREEN_WIDTH
;.rept 210
;        ; Pixel 5 clock cycles
;        ;nop                                         ; (1)
;        nop                                         ; (1)
;        nop                                         ; (1)
;        nop                                         ; (1)
;        mov.b [W7++], [W8]                          ; (1) Copy out of the previously filled scanline buffer to PORTB
;        nop                                         ; (1)
;
;.endr
;goto FINISHED_RENDERING_LINE

;12 tiles already drawn, need 19 more
; draws 76 pixels (4 pixels per loop)
.rept 19
	add W1, W3, W4                              ; (1) add current tile location to tile offset
	mov.b [W4], W5                              ; (1) get tile number       
    mov.b [W7++], [W8]   ;ok                    ; (2) Copy out of the previously filled scanline buffer to PORTB 
	sl W5, #6, W4                               ; (1) *64 since each tile is 64 bytes 
        add W4, W2, W4                              ; (1) add tile number to tile offset
	add W1, #1, W1				    ; (1) advance to new tile
	and W1, #0x1F, W1                           ; (1) mask tile number so it is always 0-31
    mov.b [W7++], [W8]  ;ok                     ; (1) Copy out of the previously filled scanline buffer to PORTB        
	nop					    ; (1)
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
    mov.b [W7++], [W8]   ;ok                    ; (1) Copy out of the previously filled scanline buffer to PORTB        
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
    mov.b [W7++], [W8]   ;ok                    ; (1) Copy out of the previously filled scanline buffer to PORTB        
        mov.b [W4++], [W6++]                        ; (1) draw pixel to scanline buffer
.endr

;134 pixels left to draw
.rept 134 ; Pixel 5 clock cycles
        nop                                         ; (1)
        nop                                         ; (1)
        nop                                         ; (1)
        mov.b [W7++], [W8]                          ; (1) Copy out of the previously filled scanline buffer to PORTB
        nop                                         ; (1)

.endr

        nop ;compensate for goto of odd rendering line
        nop
        
FINISHED_RENDERING_LINE:
        ; Now set PORTB rgb components to 0
        mov #0xFFC0, W0                             ; (1) Load up our mask
        and LATB, WREG                              ; (1) mask off all color bits to 0
        mov WREG, PORTB                             ; (1) write out zeros

        ; Now increment our framepointer if we are done with this line (every 2 lines = 240 horizontal lines)
        dec ReptLine                                ; (1)
        bra nz, FINISHED_VGA                        ; (1/2)

        ; Increment our tile line count by 8 until we hit 64
        mov #8, W0                                  ; (1) 
        add LINECOUNT                               ; (1) Linecount += 8

        ; Increment our raster line count
        inc _g_RasterLine                           ; (1) g_RasterLine++

        ; Check to see if we hit 64
        mov #64, W0                                 ; (1)
        cp LINECOUNT                                ; (1)
        bra nz, NOT_NEW_TILE_ROW                    ; (1/2) if (Linecount != 64) jmp

        clr LINECOUNT                               ; (1) clear our tile line counter
        mov _g_TileMapLogicalWidth, W0              ; (1) W0 = Logical width of tile map
        add TILEINDPTR, WREG                        ; (1) W0 += TileIndexPointer
        mov W0, TILEINDPTR                          ; (1) save it out


NOT_NEW_TILE_ROW:

        ; Swap our filling and drawing buffers
        mov FILLBUFPTR, W0                          ; (1)
        mov DRAWBUFPTR, W1                          ; (1)
        mov W0, DRAWBUFPTR                          ; (1)
        mov W1, FILLBUFPTR                          ; (1) 

        mov #LINE_REPT_COUNT, W0                    ; (1) 
        mov W0, ReptLine                            ; (1) Reset our line repeate counter
        

        ; Done with this completely
FINISHED_VGA:

        BCLR IFS0, #T1IF                            ; Clear the Timer1 Interrupt flag Status
                                                    ; bit.

        pop RCOUNT
        pop W12
        pop W11
        pop W10
        pop W9
        pop W8
        pop W7
        pop W6
        pop W5
        pop W4
        pop W3
        pop W2
        pop W1
        pop W0                                      ; Retrieve context POP-ping from Stack
        pop.s                                       ; (1) Push status registers

        disi #0x0                                   ; Re-enable interrupts

        retfie                                      ;Return from Interrupt Service routine




; ------------------- ODD SCANLINE RENDERING PATH --------------------------------------
ODD_LINE_RENDER:

        ; Total clocks spent in hsync at this point is 22
        ; DO NOT USE OR TOUCH W7 or W8 in here (used for rendering scanline backbuffer to VGA)!!!!

        ; ---------- Rendering Space -------------------- 
        repeat #(152-5-65-64)                                  ; (1)
        nop                                         ; (1 + n)              
		;set up scanline for all sprites (done once per scanline)
        mov _g_SpriteList, W2; (1) pointer for sprite data        
        mov _g_RasterLine, W5; (1) find out which scanline we are using
        add W5, #16, W5 ; (1) add this so we can move sprite offscreen on the Y axis
        mov #0, W4     ; (1) make sure that upper 8 bits are clear for sprite Y value
        mov #0xFF, W10    ; (1) used for compare for sprite
        ;---------------------------------
		Draw_Sprite ;(65) - draws 1st sprite
		Draw_Sprite ;(64) - draws 2nd sprite

        ; ------- HSYNC OFF ---------
        bclr LATB, #VGA_HSYNC_BIT                   ; (1) turn hsync bit off


        ; ------ LEFT BLANKING REGION --------
        ; Now set PORTB rgb components to 0 (we should spend about 80 clock cycles here) 
        clr PORTB                                   ; (1) draw black


        ; ---------- Rendering Space -------------------- 
        ; Free rendering space
        repeat #(72-3-64)                                  ; (1)
        nop                                         ; (1 + n)
        Draw_Sprite ;(64) - draws 3rd sprite
        ; Render out the scanline
        ;add #16, W6                                  ; (1) Increment past boarder
        add #16, W7                                  ; (1) Increment past boarder
	    mov #0, W12  ;(1) black, used if fake scanlines are enabled
;1056 cycles
;210 pixels
		nop;	
		Draw_Sprite_Signal ;(108) 21 pixels - draws sprite during signal
		Draw_Sprite_Signal ;(108) 21 pixels - draws sprite during signal
		Draw_Sprite_Signal ;(108) 21 pixels - draws sprite during signal
		Draw_Sprite_Signal ;(108) 21 pixels - draws sprite during signal
		Draw_Sprite_Signal ;(108) 21 pixels - draws sprite during signal

;.rept VGA_SCREEN_WIDTH
.rept (210-(21*5))
        ; Pixel 5 clock cycles
        nop                                         ; (1)
        nop                                         ; (1)
        nop                                         ; (1)
        Draw_Odd_Scanline ;(1) - Macro
        nop                                         ; (1)
.endr

        goto FINISHED_RENDERING_LINE
        

;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
        